
/* WWW CGI Blackjack player  -- Henry Minsky (hqm@ai.mit.edu) 
 *
 * $Header: /usr/local/etc/httpd/htdocs/FreeSoft/blackjack/RCS/bj.h,v 1.5 1995/12/02 04:29:48 baccala Exp baccala $
 *
 * This program is the property of Universal Access Inc. 
 *
 *     Copyright (C) 1994 Henry Minsky
 *     
 *     This program is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU General Public License
 *     as published by the Free Software Foundation; either version 2
 *     of the License, or (at your option) any later version.
 *     
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *     
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * */


#define SEPCHAR '@'


#define MAXDECKS 10


typedef enum {
  _NOVAL_, ACE, TWO, THREE, FOUR, FIVE, SIX, SEVEN, EIGHT, NINE, TEN, JACK, QUEEN, KING
} Value;


typedef enum {
  _NOSUIT_, CLUBS, DIAMONDS, HEARTS, SPADES
 } Suit;

typedef struct {
  Value val;
  Suit suit;
} Card;

typedef enum { IN_PLAY, BUST, CLOSED, WON, LOST, PUSH, SURRENDERED, 
		 SPLIT_PAIR, DOUBLED, BLACKJACK } State;

typedef enum { IDLE, STAND, HIT, SPLIT, DOUBLE, SURRENDER } Move;

typedef struct {
  State state;
  char *name;
} StateMap;

typedef struct {
  Move move;
  char *name;
} MoveMap;

typedef struct {
  Value val;
  char c;
} ValueMap;

typedef struct {
  Suit s;
  char c;
} SuitMap;



typedef struct {
  State state;		/* Values: IN_PLAY BUST CLOSED WON LOST PUSH
                                   SURRENDERED BLACKJACK */
  State aux;		/* Values: IN_PLAY SPLIT_PAIR DOUBLED */

  int bet; 
  Move move;
  int softvalue;
  int value;
  int ncards;
  Card cards[64];
  int insurance;
} Hand;

typedef struct {
  Hand hands[32];
  int nhands;
  int balance;
  int hands_closed;
  char *options;
  int seqnum;
  int ndealt; /* ncards dealt in this shoe so far */
  int seed;
  Card dealer_upcard;
  Card dealer_holecard;
  int bet;

  /* Used to compute the delta amount of money won or lost this hand. */
  int orig_balance;   
  

} Player;



typedef struct {
  int ncards; /* How many cards in the deck */
  int n;  /* The current card */
  int shuffled;
  Card cards[MAXDECKS*52];

} Deck;


  
/****************************************************************/

char *face_name(Value val);
char *suit_name(Suit suit);
void print_card(Card card);

void play_hand(Player *p, int nhand);

Card deal_card ( Hand *hand, Deck *deck) ;
Deck * new_deck(int ndecks);

int card_value_soft (Card card);
int card_value_hard (Card card);

Card makecard(Value val, Suit suit);

void evaluate_hand (Hand *hand);

void init_hand(Hand *h, int bet);
void shuffle_deck(Deck *deck);
Suit lookup_cardsuit(char c);
Value lookup_cardval(char c);
Card parse_card(char *token);
void parse_hand(Player *p, char *line);

State parse_state(char *token);
Move parse_move(char *token);

Move name2move(char *name);
char *move2name(Move s);
State name2state(char *name);
char *state2name(State s);

void format_hands(Player *p, FILE *out);
void format_cards(Hand *h, FILE *out);
void format_chips(FILE *, int);

void print_hand ( Hand h, FILE *out);
char *format_card(Card c);

char suit2char (Suit s);
char value2char (Value s);

void plustospace(char *str);

void unescape_url(char *url);
void spacetoplus(char *str);

Value char2value (char c);

void evaluate_player(Player *p);
void play_dealer_hand(Player *p);
void  format_next_move(Player *p, FILE *out);

void format_final_page(Player *p, FILE *out);
void format_next_page(Player *p, FILE *out);
void format_hand(Player *p, Hand *h, FILE *out, int chips);


char *pretty_format_card(Card c);

void evaluate_final_hand(Player *p);
void format_hand_options(Player *p, Hand *h, FILE *out);

int has_ace(Hand *hand);
char * card_inline_image(Card c);

char *state2pretty_name(State s);
